#include <string>
#include <format>
#include <vector>
#include <chrono>
#include "mcolor.h"
#include "jlog/formatter.hpp"
#include "jlog/token.hpp"

namespace jlog {
    std::string get_timestamp() {
        using namespace std::chrono;
        auto const timestamp = current_zone()->to_local(system_clock::now());
        auto dp = floor<days>(timestamp);
        year_month_day ymd{floor<days>(timestamp)};
        hh_mm_ss time{floor<milliseconds>(timestamp - dp)};
        auto y = ymd.year();
        auto m = ymd.month();
        auto d = ymd.day();
        auto h = time.hours();
        auto M = time.minutes();
        auto s = time.seconds();
        auto ms = time.subseconds();
        return std::format("{}-{}-{} {}:{}:{}.{}", y, m, d, h.count(), M.count(), s.count(), ms.count());
    }

    std::vector<token> trace_format(
            const std::string& func,
            const std::string& file,
            int line)
    {
        auto trace = token{.content = func};
        auto filedata = token{.content = std::format("{}:{}", file, line)};
        return {trace, filedata};
    }

    std::vector<token> log_format(
            const std::string& severity_name,
            const std::string& message,
            const Color4& severity_cc)
    {
        auto severity = token{.colorCode = severity_cc, .content = severity_name};
        auto content = token{.content = message, .delimiter = ""};
        return {severity, content};
    }

    std::vector<token> log_format(
            const std::string& severity_name,
            const std::string& message,
            const std::string& func,
            const std::string& file,
            int line,
            const Color4& severity_cc)
    {
        std::vector<token> tokens;
        auto timestamp = token{.content = jlog::get_timestamp()};
        auto tf_tokens = jlog::trace_format(func, file, line);
        auto lf_tokens = jlog::log_format(severity_name, message, severity_cc);
        tokens.push_back(timestamp);
        tokens.insert(tokens.end(), tf_tokens.begin(), tf_tokens.end());
        tokens.insert(tokens.end(), lf_tokens.begin(), lf_tokens.end());
        return tokens;
    }
}