#include <string>
#include <vector>
#include <functional>
#include <format>
#include <mcolor.h>
#include <jlog/token.hpp>

namespace jlog {
    std::string toks2msg(std::vector <token> tokens, std::function<std::string(token)> formatter) {
        std::string msg;
        for (const token &t: tokens) {
            msg += formatter(t);
        }
        return msg;
    }

    std::string toks2consoleMsg(std::vector <token> tokens) {
        return toks2msg(tokens, consoleMsgFormatter);
    }

    std::string toks2logfileMsg(std::vector <token> tokens) {
        return toks2msg(tokens, logfileMsgFormatter);
    }

    std::string consoleMsgFormatter(token t) {
        if (!t.delimiter.empty()) {
            return std::format("{}{}{}{}{} ", mcolor::toEscapeCode(t.colorCode), t.delimiter[0], t.content,
                               t.delimiter[1], mcolor::toEscapeCode(AnsiColor::RESET));
        }

        return std::format("{}{}{} ", mcolor::toEscapeCode(t.colorCode), t.content,
                           mcolor::toEscapeCode(AnsiColor::RESET));
    }

    std::string logfileMsgFormatter(token t) {
        if (!t.delimiter.empty()) {
            return std::format("{}{}{} ", t.delimiter[0], t.content, t.delimiter[1]);
        }

        return t.content + " ";
    }
}